"""
Auth hooks
"""

# Alliance Auth
from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook

# AA Fleet Finder
from fleetfinder import __title_translated__, urls


class FleetFinderMenuItem(MenuItemHook):  # pylint: disable=too-few-public-methods
    """
    This class ensures only authorized users will see the menu entry
    """

    def __init__(self):
        # Setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            text=__title_translated__,
            classes="fa-solid fa-users",
            url_name="fleetfinder:dashboard",
            navactive=["fleetfinder:"],
        )

    def render(self, request):
        """
        Render app pages
        :param request:
        :return:
        """

        if request.user.has_perm("fleetfinder.access_fleetfinder"):
            return MenuItemHook.render(self, request=request)

        return ""


@hooks.register("menu_item_hook")
def register_menu():
    """
    Register our menu
    :return:
    """

    return FleetFinderMenuItem()


@hooks.register("url_hook")
def register_urls():
    """
    Register our urls
    :return:
    """

    return UrlHook(urls=urls, namespace="fleetfinder", base_url="^fleetfinder/")



"""
Test checks for access to fleetfinder
"""

# Standard Library
from http import HTTPStatus

# Django
from django.contrib.auth.models import Group
from django.urls import reverse

# AA Fleet Finder
from fleetfinder.tests import BaseTestCase
from fleetfinder.tests.utils import create_fake_user


class TestAccess(BaseTestCase):
    """
    Testing module access
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

        # User cannot access fleetfinder
        cls.user_1001 = create_fake_user(
            character_id=1001, character_name="Peter Parker"
        )

        # User can access fleetfinder
        cls.user_1002 = create_fake_user(
            character_id=1002,
            character_name="Bruce Wayne",
            permissions=["fleetfinder.access_fleetfinder"],
        )

    def test_has_no_access(self):
        """
        Test that a user without access gets a 302

        :return:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(path=reverse(viewname="fleetfinder:dashboard"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_has_access(self):
        """
        Test that a user with access gets to see it

        :return:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(path=reverse(viewname="fleetfinder:dashboard"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)



"""
ESI Handler Module for AA Fleetfinder
"""

# Standard Library
from typing import Any

# Third Party
from aiopenapi3 import ContentTypeError
from httpx import Response

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger
from esi.exceptions import HTTPClientError, HTTPNotModified
from esi.openapi_clients import EsiOperation

# AA Fleet Finder
from fleetfinder import __title__
from fleetfinder.providers import AppLogger

logger = AppLogger(my_logger=get_extension_logger(__name__), prefix=__title__)


def result(  # pylint: disable=too-many-arguments too-many-positional-arguments
    operation: EsiOperation,
    use_etag: bool = True,
    return_response: bool = False,
    force_refresh: bool = False,
    use_cache: bool = True,
    **extra,
) -> tuple[Any, Response] | Any:
    """
    Retrieve the result of an ESI operation, handling HTTPNotModified exceptions.

    :param operation: The ESI operation to execute.
    :type operation: EsiOperation
    :param use_etag: Whether to use ETag for caching.
    :type use_etag: bool
    :param return_response: Whether to return the full response object.
    :type return_response: bool
    :param force_refresh: Whether to force a refresh of the data.
    :type force_refresh: bool
    :param use_cache: Whether to use cached data.
    :type use_cache: bool
    :param extra: Additional parameters to pass to the operation.
    :type extra: dict
    :return: The result of the ESI operation, optionally with the response object.
    :rtype: tuple[Any, Response] | Any
    """

    logger.debug(f"Handling ESI operation: {operation.operation.operationId}")

    try:
        esi_result = operation.result(
            use_etag=use_etag,
            return_response=return_response,
            force_refresh=force_refresh,
            use_cache=use_cache,
            **extra,
        )
    except HTTPNotModified:
        logger.debug(
            f"ESI returned 304 Not Modified for operation: {operation.operation.operationId} - Skipping update."
        )

        esi_result = None
    except ContentTypeError:
        logger.warning(
            msg="ESI returned gibberish (ContentTypeError) - Skipping update."
        )

        esi_result = None
    except HTTPClientError as exc:
        logger.error(
            msg=f"Error while fetching data from ESI for operation: {operation.operation.operationId} -  {str(exc)}"
        )

        # Re-raise the exception to be handled by the caller
        raise

    return esi_result
